import { createContext, useContext, useEffect, useRef, useMemo } from 'react';

const isServerSide = typeof document === 'undefined';

const META = 'M';
const TITLE = 'T';
const LINK = 'L';
const TEMPLATE = 'P';
const SCRIPT = 'S';

const applyTitleTemplate = (title, template) => template ? template.replace(/%s/g, title || '') : title;

const changeOrCreateMetaTag = meta => {
  const result = document.head.querySelectorAll(meta.charset ? `meta[${meta.keyword}]` : `meta[${meta.keyword}="${meta[meta.keyword]}"]`);

  if (result[0]) {
    if (meta.charset) {
      result[0].setAttribute(meta.keyword, meta.charset);
    } else {
      result[0].setAttribute('content', meta.content);
    }
  } else {
    const metaTag = document.createElement('meta');

    if (meta.charset) {
      metaTag.setAttribute(meta.keyword, meta.charset);
    } else {
      metaTag.setAttribute(meta.keyword, meta[meta.keyword]);
      metaTag.setAttribute('content', meta.content);
    }

    document.head.appendChild(metaTag);
  }
};

const createDispatcher = () => {
  let lang;
  let linkQueue = [];
  let scriptQueue = [];
  let titleQueue = [];
  let titleTemplateQueue = [];
  let metaQueue = [];
  let currentTitleIndex = 0;
  let currentTitleTemplateIndex = 0;
  let currentMetaIndex = 0;

  const processQueue = (() => {
    let timeout;
    return () => {
      clearTimeout(timeout);
      timeout = setTimeout(() => {
        timeout = null;
        const visited = new Set();
        document.title = applyTitleTemplate(titleQueue[0], titleTemplateQueue[0]);
        metaQueue.forEach(meta => {
          if (!visited.has(meta.charset ? meta.keyword : meta[meta.keyword])) {
            visited.add(meta.charset ? meta.keyword : meta[meta.keyword]);
            changeOrCreateMetaTag(meta);
          }
        });
        currentTitleIndex = currentTitleTemplateIndex = currentMetaIndex = 0;
      }, 1000 / 60);
    };
  })();

  return {
    _setLang: l => {
      lang = l;
    },
    _addToQueue: (type, payload) => {
      if (!isServerSide) processQueue();

      if (type === SCRIPT) {
        scriptQueue.push(payload);
      } else if (type === TITLE) {
        titleQueue.splice(currentTitleIndex++, 0, payload);
      } else if (type === TEMPLATE) {
        titleTemplateQueue.splice(currentTitleTemplateIndex++, 0, payload);
      } else if (type === META) {
        metaQueue.splice(currentMetaIndex++, 0, payload);
      } else {
        linkQueue.push(payload);
      }
    },
    _removeFromQueue: (type, payload) => {
      if (type === TITLE || type === TEMPLATE) {
        const queue = type === TEMPLATE ? titleTemplateQueue : titleQueue;
        const index = queue.indexOf(payload);
        queue.splice(index, 1);
        if (index === 0) document.title = applyTitleTemplate(titleQueue[0] || '', titleTemplateQueue[0]);
      } else {
        const oldMeta = metaQueue[metaQueue.indexOf(payload)];

        if (oldMeta) {
          metaQueue.splice(metaQueue.indexOf(payload), 1);
          const newMeta = metaQueue.find(m => m.keyword === oldMeta.keyword && (m.charset || m[m.keyword] === oldMeta[m.keyword]));

          if (newMeta) {
            changeOrCreateMetaTag(newMeta);
          } else {
            const result = document.head.querySelectorAll(oldMeta.charset ? `meta[${oldMeta.keyword}]` : `meta[${oldMeta.keyword}="${oldMeta[oldMeta.keyword]}"]`);
            document.head.removeChild(result[0]);
          }
        }
      }
    },
    _change: (type, prevPayload, payload) => {
      if (type === TITLE || type === TEMPLATE) {
        const queue = type === TEMPLATE ? titleTemplateQueue : titleQueue;
        queue[queue.indexOf(prevPayload)] = payload;

        if (queue.indexOf(payload) === 0) {
          document.title = applyTitleTemplate(queue[queue.indexOf(payload)], titleTemplateQueue[0]);
        }
      } else {
        changeOrCreateMetaTag(metaQueue[metaQueue.indexOf(prevPayload)] = payload);
      }
    },
    _reset:  undefined,
    toStatic: () => {
      const title = applyTitleTemplate(titleQueue[titleQueue.length - 1], titleTemplateQueue[titleTemplateQueue.length - 1]);
      const visited = new Set();
      const links = [...linkQueue];
      const scripts = [...scriptQueue];
      metaQueue.reverse();
      const metas = [...metaQueue].filter(meta => {
        if (!visited.has(meta.charset ? meta.keyword : meta[meta.keyword])) {
          visited.add(meta.charset ? meta.keyword : meta[meta.keyword]);
          return true;
        }
      });
      titleQueue = [];
      titleTemplateQueue = [];
      metaQueue = [];
      linkQueue = [];
      scriptQueue = [];
      currentTitleIndex = currentTitleTemplateIndex = currentMetaIndex = 0;
      return {
        lang,
        title,
        links,
        scripts,
        metas: metas.map(meta => meta.keyword === 'charset' ? {
          charset: meta[meta.keyword]
        } : {
          [meta.keyword]: meta[meta.keyword],
          content: meta.content
        })
      };
    }
  };
};
const defaultDispatcher = createDispatcher();
const DispatcherContext = createContext(defaultDispatcher);

const useLang = language => {
  const dispatcher = useContext(DispatcherContext);

  if (isServerSide) {
    dispatcher._setLang(language);
  }

  useEffect(() => {
    document.getElementsByTagName('html')[0].setAttribute('lang', language);
  }, [language]);
};

const useLink = options => {
  const dispatcher = useContext(DispatcherContext);
  const hasMounted = useRef(false);
  const node = useRef();
  const originalOptions = useRef();

  if (isServerSide && !hasMounted.current) {
    dispatcher._addToQueue(LINK, options);
  }

  useEffect(() => {
    if (hasMounted.current) {
      Object.keys(options).forEach(key => {
        node.current.setAttribute(key, options[key]);
      });
    }
  }, [options.href, options.media, options.as, options.rel, options.crossorigin, options.type, options.hreflang]);
  useEffect(() => {
    hasMounted.current = true;
    const preExistingElements = document.querySelectorAll(`link[rel="${options.rel}"]`);
    preExistingElements.forEach(x => {
      let found = true;
      Object.keys(options).forEach(key => {
        if (x.getAttribute(key) !== options[key]) {
          found = false;
        }
      });

      if (found) {
        node.current = x;
      }
    });

    if (node.current) {
      originalOptions.current = Object.keys(options).reduce((acc, key) => {
        acc[key] = node.current.getAttribute(key);
        return acc;
      }, {});
    } else {
      node.current = document.createElement('link');
      Object.keys(options).forEach(key => {
        node.current.setAttribute(key, options[key]);
      });
      document.head.appendChild(node.current);
    }

    return () => {
      hasMounted.current = false;

      if (originalOptions.current) {
        Object.keys(originalOptions.current).forEach(key => {
          node.current.setAttribute(key, originalOptions.current[key]);
        });
      } else {
        document.head.removeChild(node.current);
      }
    };
  }, []);
};

const useScript = options => {
  const dispatcher = useContext(DispatcherContext);

  if (isServerSide) {
    dispatcher._addToQueue(SCRIPT, options);
  }

  useEffect(() => {
    const preExistingElements = options.id ? document.querySelectorAll(`script[id="${options.id}"]`) : document.querySelectorAll(`script[src="${options.src}"]`);
    let script;

    if (!preExistingElements[0] && (options.src || options.id)) {
      const _script = document.createElement('script');

      if (options.type) _script.type = options.type;
      if (options.module) _script.type = 'module';
      if (options.integrity) _script.setAttribute('integrity', options.integrity);
      if (options.crossorigin) _script.setAttribute('crossorigin', options.crossorigin);
      if (options.defer) _script.setAttribute('defer', 'true');else if (options.async) _script.setAttribute('async', 'true');
      if (options.id) _script.id = options.id;
      if (options.src) _script.src = options.src;
      if (options.text) _script.text = options.text;
      document.head.appendChild(_script);
    } else if (preExistingElements[0]) {
      script = preExistingElements[0];
    }

    return () => {
      if (script) {
        document.head.removeChild(script);
      }
    };
  }, []);
};

function extractKeyword(meta) {
  return meta.charset ? 'charset' : meta.name ? 'name' : meta.property ? 'property' : 'http-equiv';
}
const useHead = ({
  title,
  metas,
  language
}) => {
  const dispatcher = useContext(DispatcherContext);
  const hasMounted = useRef(false);
  const prevTitle = useRef();
  const prevMetas = useRef();
  const addedMetas = useRef();
  const memoizedMetas = useMemo(() => {
    const calculatedMetas = (metas || []).map(meta => {
      const keyword = extractKeyword(meta);

      if (prevMetas.current) {
        const found = prevMetas.current.find(x => x.keyword === keyword && x.name === meta.name && x.charset === meta.charset && x['http-equiv'] === meta.httpEquiv && x.property === meta.property && x.content === meta.content);
        if (found) return found;
      }

      return {
        keyword,
        name: meta.name,
        charset: meta.charset,
        'http-equiv': meta.httpEquiv,
        property: meta.property,
        content: meta.content
      };
    });
    return calculatedMetas;
  }, [metas]);

  if (isServerSide && !hasMounted.current) {
    if (title) dispatcher._addToQueue(TITLE, title);
    if (language) dispatcher._setLang(language);
    memoizedMetas.forEach(meta => {
      dispatcher._addToQueue(META, meta);
    });
  }

  useEffect(() => {
    if (prevMetas.current) {
      const previousMetas = [...prevMetas.current];
      const added = [];
      memoizedMetas.forEach(meta => {
        added.push(meta);

        if (previousMetas.includes(meta)) {
          previousMetas.splice(previousMetas.indexOf(meta), 1);
        } else {
          const previousIteration = previousMetas.find(x => x.keyword === meta.keyword && meta[meta.keyword] === x[meta.keyword]);

          if (previousIteration) {
            dispatcher._change(META, previousIteration, meta);
          } else {
            dispatcher._addToQueue(META, meta);
          }
        }
      });

      if (previousMetas.length) {
        previousMetas.forEach(meta => {
          dispatcher._removeFromQueue(META, meta);
        });
      }

      addedMetas.current = added;
      prevMetas.current = memoizedMetas;
    }
  }, [memoizedMetas]);
  useEffect(() => {
    memoizedMetas.forEach(meta => {
      dispatcher._addToQueue(META, meta);
    });
    prevMetas.current = addedMetas.current = memoizedMetas;
    return () => {
      (addedMetas.current || []).forEach(meta => {
        dispatcher._removeFromQueue(META, meta);
      });
    };
  }, []);
  useEffect(() => {
    if (hasMounted.current && title) {
      if (prevTitle.current != null) {
        dispatcher._change(TITLE, prevTitle.current, prevTitle.current = title);
      } else {
        dispatcher._addToQueue(TITLE, prevTitle.current = title);
      }
    } else if (hasMounted.current && prevTitle.current) {
      dispatcher._removeFromQueue(TITLE, prevTitle.current);

      prevTitle.current = undefined;
    }
  }, [title]);
  useEffect(() => {
    hasMounted.current = true;

    dispatcher._addToQueue(TITLE, prevTitle.current = title);

    return () => {
      hasMounted.current = false;
      if (prevTitle.current != null) dispatcher._removeFromQueue(TITLE, prevTitle.current);
    };
  }, []);
  useEffect(() => {
    if (language) document.getElementsByTagName('html')[0].setAttribute('lang', language);
  }, [language]);
};

const useMeta = options => {
  const dispatcher = useContext(DispatcherContext);
  const hasMounted = useRef(false);
  const keyword = useRef();
  const metaObject = useRef({
    keyword: keyword.current = extractKeyword(options),
    name: options.name,
    charset: options.charset,
    'http-equiv': options.httpEquiv,
    property: options.property,
    content: options.content
  });

  if (isServerSide && !hasMounted.current) {
    dispatcher._addToQueue(META, metaObject.current);
  }

  useEffect(() => {
    if (hasMounted.current) {
      dispatcher._change(META, metaObject.current, metaObject.current = {
        keyword: keyword.current,
        name: options.name,
        charset: options.charset,
        'http-equiv': options.httpEquiv,
        property: options.property,
        content: options.content
      });
    }
  }, [options.content]);
  useEffect(() => {
    dispatcher._addToQueue(META, metaObject.current);

    hasMounted.current = true;
    return () => {
      hasMounted.current = false;

      dispatcher._removeFromQueue(META, metaObject.current);
    };
  }, []);
};

const useTitle = (title, template) => {
  const dispatcher = useContext(DispatcherContext);
  const hasMounted = useRef(false);
  const prevTitle = useRef();

  if (isServerSide && !hasMounted.current) {
    dispatcher._addToQueue(template ? TEMPLATE : TITLE, title);
  }

  useEffect(() => {
    if (hasMounted.current) {
      dispatcher._change(template ? TEMPLATE : TITLE, prevTitle.current, prevTitle.current = title);
    }
  }, [title, template]);
  useEffect(() => {
    hasMounted.current = true;

    dispatcher._addToQueue(template ? TEMPLATE : TITLE, prevTitle.current = title);

    return () => {
      hasMounted.current = false;

      dispatcher._removeFromQueue(template ? TEMPLATE : TITLE, prevTitle.current);
    };
  }, [template]);
};

const useTitleTemplate = template => {
  useTitle(template, true);
};

const toStatic = defaultDispatcher.toStatic;
const HoofdProvider = DispatcherContext.Provider;

export { HoofdProvider, createDispatcher, toStatic, useHead, useLang, useLink, useMeta, useScript, useTitle, useTitleTemplate };
//# sourceMappingURL=hoofd.modern.js.map
